﻿using System;
using System.Windows;
using System.Windows.Controls;
using TestMapSuiteWPF.Components;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;


namespace TestShapeFile
{
   
   
    /// <summary>
    /// Interaction logic for MapControl.xaml
    /// </summary>
    public partial class MapControl : UserControl
    {
        public MapControl()
        {
            InitializeComponent();
        }

        LayerOverlay worldOverlay;
        private void wpfMap1_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.Meter;

            ManagedProj4Projection proj4 = new ManagedProj4Projection();
            proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4.ExternalProjectionParametersString = Proj4Projection.GetSphericalMercatorParametersString();


            MyManagedProj4Projection myproj = new MyManagedProj4Projection();
            myproj.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            myproj.ExternalProjectionParametersString = Proj4Projection.GetSphericalMercatorParametersString();


            string sMapShapeFileName = @"e:\testdata\countries02.shp";
            //string sMapShapeIndexFileName = @"..\countries02.idx";

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(sMapShapeFileName);
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.InternationalBorder2;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.FeatureSource.Projection = proj4;

            InMemoryFeatureLayer fLayer = new InMemoryFeatureLayer();
            fLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            fLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            fLayer.FeatureSource.Projection = proj4;
            PointShape point = new PointShape();
            point.X = -71.55;
            point.Y = -32.75;
            fLayer.InternalFeatures.Add(new Feature(point));

            worldOverlay = new LayerOverlay();
            worldOverlay.Layers.Add("WorldLayer", worldLayer);
            worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            worldOverlay.TileType = TileType.MultipleTile;
            wpfMap1.Overlays.Add(worldOverlay);
            worldLayer.Open();
            worldLayer.WrappingExtent = worldLayer.GetBoundingBox();
            worldLayer.WrappingMode = WrappingMode.WrapDateline;
            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.Layers.Add("TestLayer", fLayer);
            staticOverlay.WrappingMode = WrappingMode.WrapDateline;
            wpfMap1.Overlays.Add(staticOverlay);


            wpfMap1.CurrentExtent = worldLayer.GetBoundingBox();
            string cacheDirectory = AppDomain.CurrentDomain.BaseDirectory + @"Resources\Tiles1\";
            string cacheId = "mercator";
            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();
            bitmapTileCache.ClearCache();
            bitmapTileCache.CacheDirectory = cacheDirectory;
            bitmapTileCache.CacheId = cacheId;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;

            // we need set the bounding box unit to meter here. 
            // Although, we will set this property on our side, 
            // but setting this property will restore the property to its default values,
            // so that the following bounding box won't be set properly.
            bitmapTileCache.TileMatrix.BoundingBoxUnit = GeographyUnit.Meter;

            // the wrapping date line has some limits that the max bounding box's width and height need to be n times of the real world.
            // the default max extent doesn't follow this rule. We need change it manually here as well as the method we generate the cache.
            bitmapTileCache.TileMatrix.BoundingBox = GenerateTiles.MaxExtentForWrappingDateline;
            worldOverlay.TileCache = bitmapTileCache;

            SetCustomZoomLevels(worldLayer.GetBoundingBox());

            InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

     LineShape line = new LineShape("LINESTRING(19123575.3233765 6033964.19601206,19465326.1601118 6090234.49270567,19465326.1601118 6090234.49270567,19817095.7510186 6135183.51991344,19817095.7510186 6135183.51991344,-19899472.1742056 6165281.14487133,-19899472.1742056 6165281.14487133,-19536570.6342195 6180369.8294395,-19536570.6342195 6180369.8294395,-19173669.0942334 6180369.8294395,-19173669.0942334 6180369.8294395,-18810767.5542474 6163606.27028248,-18810767.5542474 6163606.27028248,-18462337.5480644 6133514.53132379,-18462337.5480644 6133514.53132379,-18109454.7622497 6088574.22537154,-18109454.7622497 6088574.22537154,-17749892.8069875 6027368.22106761,-17749892.8069875 6027368.22106761,-17401462.8008045 5953507.34820183,-17401462.8008045 5953507.34820183,-17059711.9640692 5865692.7163159,-17059711.9640692 5865692.7163159,-16726866.6865973 5767538.41918456,-16726866.6865973 5767538.41918456,-16405153.3582047 5657803.24745032,-16405153.3582047 5657803.24745032,-16087892.8094439 5538466.20295752,-16087892.8094439 5538466.20295752,-15779537.8199465 5408361.23322365,-15779537.8199465 5408361.23322365,-15478975.1948047 5270964.9761426,-15478975.1948047 5270964.9761426,-15188431.3238342 5126588.58192275,-15188431.3238342 5126588.58192275,-14906793.0121273 4974067.03336827,-14906793.0121273 4974067.03336827,-14634060.2596837 4816656.71984445,-14634060.2596837 4816656.71984445,-14373572.6512275 4655993.08435659,-14373572.6512275 4655993.08435659,-14207706.6099455 4548393.63894527,-14207706.6099455 4548393.63894527,-14143141.3052854 4507628.33554598,-14143141.3052854 4507628.33554598,-13899351.6204481 4344742.6860991,-13899351.6204481 4344742.6860991,-13662241.1050585 4176118.52067981,-13662241.1050585 4176118.52067981,-13430696.5642085 4005997.95753182,-13430696.5642085 4005997.95753182,-13399527.1067863 3981901.84210337,-13399527.1067863 3981901.84210337,-13167982.5659363 3976554.00156425,-13167982.5659363 3976554.00156425,-13164642.9812125 3984576.69424295,-13164642.9812125 3984576.69424295,-13163529.7863046 3987252.16844898)");
     //  LineShape lineDegrees =  proj4.ConvertToInternalProjection(line) as LineShape;


            CustomStyle style = new CustomStyle();
            featureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(style);
            featureLayer.WrappingMode = WrappingMode.WrapDateline;
            featureLayer.WrappingExtent = worldLayer.GetBoundingBox();
            featureLayer.InternalFeatures.Add(new Feature(line));
       //     featureLayer.FeatureSource.Projection = proj4;
            featureLayer.DrawingMarginPercentage = 30;
            LayerOverlay overlay1 = new LayerOverlay();
            overlay1.TileType = TileType.SingleTile;
            overlay1.WrappingMode = WrappingMode.WrapDateline;
            overlay1.WrappingExtent = worldLayer.GetBoundingBox();
            overlay1.Layers.Add(featureLayer);
            wpfMap1.Overlays.Add(overlay1);



            //GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            //graticuleAdornmentLayer.GraticuleLineStyle = LineStyles.DegreeLine1;
            //graticuleAdornmentLayer.WrappingMode = WrappingMode.WrapDateline;
            //graticuleAdornmentLayer.WrappingExtent = worldLayer.GetBoundingBox();

            //wpfMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);
          //  GenerateTiles.Generate(wpfMap1.ZoomLevelSet, worldLayer, wpfMap1.MapUnit, m_TileSize, m_TileSize, cacheDirectory, cacheId);

            wpfMap1.Refresh();
        }


        private const double MetersInInch = 39.3701;
        private int m_TileSize = 256;
        private double m_MaxScale = 0;
        private void SetCustomZoomLevels(RectangleShape WorldExtent)
        {
            m_TileSize = GetTileSize();
            m_TileSize = 256;
            int drawWidth = Convert.ToInt32(Math.Floor(this.ActualWidth / m_TileSize)) * m_TileSize;
            //drawWidth -= 1;

            double resolution = WorldExtent.Width / drawWidth;
            double scaleMax = resolution * MetersInInch * 96;
            m_MaxScale = scaleMax;
            for (int i = 0; i < 15; i++)
            {
                wpfMap1.ZoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(scaleMax));
                scaleMax = scaleMax / 2;
            }

            MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(m_MaxScale, m_TileSize, m_TileSize, wpfMap1.MapUnit, WorldExtent);
            worldOverlay.TileCache.TileMatrix = tileMatrix;
            worldOverlay.TileHeight = worldOverlay.TileWidth = m_TileSize;
        }

        private int GetTileSize()
        {
            double screenWidth = this.ActualWidth;
            int tileSize = -1;
            for (double i = 5; i <= 10; i++)
            {
                if (screenWidth % i == 0)
                {
                    tileSize = (int)(screenWidth / i);
                    break;
                }
            }
            if (tileSize == -1)
            {
                for (double i = 4; i <= 2; i--)
                {
                    if (screenWidth % i == 0)
                    {
                        tileSize = (int)(screenWidth / i);
                        break;
                    }
                }
            }

            if (tileSize == -1)
            {
                for (double i = 11; i <= 20; i++)
                {
                    if (screenWidth % i == 0)
                    {
                        tileSize = (int)(screenWidth / i);
                        break;
                    }
                }
            }
            if (tileSize == -1)
            {
                tileSize = (int)Math.Floor(screenWidth);
            }

            return tileSize;

        }
    }
}
